Class: File
- Inherits:
-
Object
- Object
- File
- Defined in:
- lib/ptools.rb
Constant Summary collapse
- PTOOLS_VERSION =
The version of the ptools library.
'1.4.2'.freeze
- IMAGE_EXT =
%w[.bmp .gif .jpg .jpeg .png .ico]
Class Method Summary collapse
-
.binary?(file, percentage = 0.30) ⇒ Boolean
Returns whether or not
fileis a binary non-image file, i.e. -
.bmp?(file) ⇒ Boolean
Is the file a bitmap file?.
-
.gif?(file) ⇒ Boolean
Is the file a gif?.
-
.head(filename, num_lines = 10) ⇒ Object
In block form, yields the first
num_linesfromfilename. -
.ico?(file) ⇒ Boolean
Is the file an ico file?.
-
.image?(file, check_file_extension = true) ⇒ Boolean
Returns whether or not the file is an image.
-
.jpg?(file) ⇒ Boolean
Is the file a jpeg file?.
-
.nl_convert(old_file, new_file = old_file, platform = 'local') ⇒ Object
Converts a text file from one OS platform format to another, ala ‘dos2unix’.
-
.nl_for_platform(platform) ⇒ Object
Returns the newline characters for the given platform.
-
.png?(file) ⇒ Boolean
Is the file a png file?.
-
.sparse?(file) ⇒ Boolean
Returns whether or not
fileis a sparse file. -
.tail(filename, num_lines = 10) ⇒ Object
In block form, yields the last
num_linesof filefilename. -
.tiff?(file) ⇒ Boolean
Is the file a tiff?.
-
.touch(filename) ⇒ Object
Changes the access and modification time if present, or creates a 0 byte file
filenameif it doesn’t already exist. -
.wc(filename, option = 'all') ⇒ Object
With no arguments, returns a four element array consisting of the number of bytes, characters, words and lines in filename, respectively.
-
.whereis(program, path = ENV['PATH']) ⇒ Object
Returns an array of each
programwithinpath, or nil if it cannot be found. -
.which(program, path = ENV['PATH']) ⇒ Object
Looks for the first occurrence of
programwithinpath.
Class Method Details
.binary?(file, percentage = 0.30) ⇒ Boolean
Returns whether or not file is a binary non-image file, i.e. executable, shared object, ect. Note that this is NOT guaranteed to be 100% accurate. It performs a “best guess” based on a simple test of the first File.blksize characters, or 4096, whichever is smaller.
By default it will check to see if more than 30 percent of the characters are non-text characters. If so, the method returns true. You can configure this percentage by passing your own as a second argument.
Example:
File.binary?('somefile.exe') # => true
File.binary?('somefile.txt') # => false
– Based on code originally provided by Ryan Davis (which, in turn, is based on Perl’s -B switch).
77 78 79 80 81 82 83 84 85 86 |
# File 'lib/ptools.rb', line 77 def self.binary?(file, percentage = 0.30) return false if File.stat(file).zero? return false if image?(file) return false if check_bom?(file) bytes = File.stat(file).blksize bytes = 4096 if bytes > 4096 s = (File.read(file, bytes) || "") s = s.encode('US-ASCII', :undef => :replace).split(//) ((s.size - s.grep(" ".."~").size) / s.size.to_f) > percentage end |
.bmp?(file) ⇒ Boolean
Is the file a bitmap file?
453 454 455 |
# File 'lib/ptools.rb', line 453 def self.bmp?(file) IO.read(file, 3) == "BM6" end |
.gif?(file) ⇒ Boolean
Is the file a gif?
471 472 473 |
# File 'lib/ptools.rb', line 471 def self.gif?(file) ['GIF89a', 'GIF97a'].include?(IO.read(file, 6)) end |
.head(filename, num_lines = 10) ⇒ Object
213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 |
# File 'lib/ptools.rb', line 213 def self.head(filename, num_lines=10) a = [] IO.foreach(filename){ |line| break if num_lines <= 0 num_lines -= 1 if block_given? yield line else a << line end } return a.empty? ? nil : a # Return nil in block form end |
.ico?(file) ⇒ Boolean
Is the file an ico file?
500 501 502 |
# File 'lib/ptools.rb', line 500 def self.ico?(file) ["\000\000\001\000", "\000\000\002\000"].include?(IO.read(file, 4, nil, :encoding => 'binary')) end |
.image?(file, check_file_extension = true) ⇒ Boolean
Returns whether or not the file is an image. Only JPEG, PNG, BMP, GIF, and ICO are checked against.
This reads and checks the first few bytes of the file. For a version that is more robust, but which depends on a 3rd party C library (and is difficult to build on MS Windows), see the ‘filemagic’ library.
By default the filename extension is also checked. You can disable this by passing false as the second argument, in which case only the contents are checked.
Examples:
File.image?('somefile.jpg') # => true
File.image?('somefile.txt') # => false
– The approach I used here is based on information found at en.wikipedia.org/wiki/Magic_number_(programming)
50 51 52 53 54 55 56 57 58 |
# File 'lib/ptools.rb', line 50 def self.image?(file, check_file_extension = true) bool = bmp?(file) || jpg?(file) || png?(file) || gif?(file) || tiff?(file) || ico?(file) if check_file_extension bool = bool && IMAGE_EXT.include?(File.extname(file).downcase) end bool end |
.jpg?(file) ⇒ Boolean
Is the file a jpeg file?
459 460 461 |
# File 'lib/ptools.rb', line 459 def self.jpg?(file) IO.read(file, 10, nil, :encoding => 'binary') == "\377\330\377\340\000\020JFIF".force_encoding(Encoding::BINARY) end |
.nl_convert(old_file, new_file = old_file, platform = 'local') ⇒ Object
Converts a text file from one OS platform format to another, ala ‘dos2unix’. The possible values for platform include:
-
MS Windows -> dos, windows, win32, mswin
-
Unix/BSD -> unix, linux, bsd, osx, darwin, sunos, solaris
-
Mac -> mac, macintosh, apple
You may also specify ‘local’, in which case your CONFIG value will be used. This is the default.
Note that this method is only valid for an ftype of “file”. Otherwise a TypeError will be raised. If an invalid format value is received, an ArgumentError is raised.
292 293 294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 310 311 312 313 314 315 316 317 318 319 320 321 322 323 324 325 326 327 328 329 330 331 332 333 334 335 |
# File 'lib/ptools.rb', line 292 def self.nl_convert(old_file, new_file = old_file, platform = 'local') unless File::Stat.new(old_file).file? raise ArgumentError, 'Only valid for plain text files' end format = nl_for_platform(platform) orig = $\ # $OUTPUT_RECORD_SEPARATOR $\ = format if old_file == new_file require 'fileutils' require 'tempfile' begin temp_name = Time.new.strftime("%Y%m%d%H%M%S") tf = Tempfile.new('ruby_temp_' + temp_name) tf.open IO.foreach(old_file){ |line| line.chomp! tf.print line } ensure tf.close if tf && !tf.closed? end File.delete(old_file) FileUtils.mv(tf.path, old_file) else begin nf = File.new(new_file, 'w') IO.foreach(old_file){ |line| line.chomp! nf.print line } ensure nf.close if nf && !nf.closed? end end $\ = orig self end |
.nl_for_platform(platform) ⇒ Object
Returns the newline characters for the given platform.
436 437 438 439 440 441 442 443 444 445 446 447 448 449 |
# File 'lib/ptools.rb', line 436 def self.nl_for_platform(platform) platform = RbConfig::CONFIG["host_os"] if platform == 'local' case platform when /dos|windows|win32|mswin|mingw/i return "\cM\cJ" when /unix|linux|bsd|cygwin|osx|darwin|solaris|sunos/i return "\cJ" when /mac|apple|macintosh/i return "\cM" else raise ArgumentError, "Invalid platform string" end end |
.png?(file) ⇒ Boolean
Is the file a png file?
465 466 467 |
# File 'lib/ptools.rb', line 465 def self.png?(file) IO.read(file, 4, nil, :encoding => 'binary') == "\211PNG".force_encoding(Encoding::BINARY) end |
.sparse?(file) ⇒ Boolean
Returns whether or not file is a sparse file.
A sparse file is a any file where its size is greater than the number of 512k blocks it consumes, i.e. its apparent and actual file size is not the same.
See en.wikipedia.org/wiki/Sparse_file for more information.
412 413 414 415 |
# File 'lib/ptools.rb', line 412 def self.sparse?(file) stats = File.stat(file) stats.size > stats.blocks * 512 end |
.tail(filename, num_lines = 10) ⇒ Object
In block form, yields the last num_lines of file filename. In non-block form, it returns the lines as an array.
Example:
File.tail('somefile.txt') # => ['This is line7', 'This is line8', ...]
If you’re looking for tail -f functionality, please use the file-tail gem instead.
– Internally I’m using a 64 chunk of memory at a time. I may allow the size to be configured in the future as an optional 3rd argument.
243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 |
# File 'lib/ptools.rb', line 243 def self.tail(filename, num_lines=10) tail_size = 2**16 # 64k chunks # MS Windows gets unhappy if you try to seek backwards past the # end of the file, so we have some extra checks here and later. file_size = File.size(filename) read_bytes = file_size % tail_size read_bytes = tail_size if read_bytes == 0 line_sep = File::ALT_SEPARATOR ? "\r\n" : "\n" buf = '' # Open in binary mode to ensure line endings aren't converted. File.open(filename, 'rb'){ |fh| position = file_size - read_bytes # Set the starting read position # Loop until we have the lines or run out of file while buf.scan(line_sep).size <= num_lines and position >= 0 fh.seek(position, IO::SEEK_SET) buf = fh.read(read_bytes) + buf read_bytes = tail_size position -= read_bytes end } lines = buf.split(line_sep).pop(num_lines) if block_given? lines.each{ |line| yield line } else lines end end |
.tiff?(file) ⇒ Boolean
Is the file a tiff?
477 478 479 480 481 482 483 484 485 486 487 488 489 490 491 492 493 494 495 496 |
# File 'lib/ptools.rb', line 477 def self.tiff?(file) return false if File.size(file) < 12 bytes = IO.read(file, 4) # II is Intel, MM is Motorola if bytes[0..1] != 'II' && bytes[0..1] != 'MM' return false end if bytes[0..1] == 'II' && bytes[2..3].ord != 42 return false end if bytes[0..1] == 'MM' && bytes[2..3].reverse.ord != 42 return false end true end |
.touch(filename) ⇒ Object
Changes the access and modification time if present, or creates a 0 byte file filename if it doesn’t already exist.
340 341 342 343 344 345 346 347 348 |
# File 'lib/ptools.rb', line 340 def self.touch(filename) if File.exist?(filename) time = Time.now File.utime(time, time, filename) else File.open(filename, 'w'){} end self end |
.wc(filename, option = 'all') ⇒ Object
With no arguments, returns a four element array consisting of the number of bytes, characters, words and lines in filename, respectively.
Valid options are ‘bytes’, ‘characters’ (or just ‘chars’), ‘words’ and ‘lines’.
356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 392 393 394 395 396 397 398 399 400 |
# File 'lib/ptools.rb', line 356 def self.wc(filename, option='all') option.downcase! valid = %w/all bytes characters chars lines words/ unless valid.include?(option) raise ArgumentError, "Invalid option: '#{option}'" end n = 0 if option == 'lines' IO.foreach(filename){ n += 1 } return n elsif option == 'bytes' File.open(filename){ |f| f.each_byte{ n += 1 } } return n elsif option == 'characters' || option == 'chars' File.open(filename){ |f| while f.getc n += 1 end } return n elsif option == 'words' IO.foreach(filename){ |line| n += line.split.length } return n else bytes,chars,lines,words = 0,0,0,0 IO.foreach(filename){ |line| lines += 1 words += line.split.length chars += line.split('').length } File.open(filename){ |f| while f.getc bytes += 1 end } return [bytes,chars,words,lines] end end |
.whereis(program, path = ENV['PATH']) ⇒ Object
Returns an array of each program within path, or nil if it cannot be found.
On Windows, it looks for executables ending with the suffixes defined in your PATHEXT environment variable, or ‘.exe’, ‘.bat’ and ‘.com’ if that isn’t defined, which you may optionally include in program.
Examples:
File.whereis('ruby') # => ['/usr/bin/ruby', '/usr/local/bin/ruby']
File.whereis('foo') # => nil
155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 |
# File 'lib/ptools.rb', line 155 def self.whereis(program, path=ENV['PATH']) if path.nil? || path.empty? raise ArgumentError, "path cannot be empty" end paths = [] # Bail out early if an absolute path is provided. if program =~ /^\/|^[a-z]:[\\\/]/i program += WIN32EXTS if MSWINDOWS && File.extname(program).empty? program = program.tr("\\", '/') if MSWINDOWS found = Dir[program] if found[0] && File.executable?(found[0]) && !File.directory?(found[0]) if File::ALT_SEPARATOR return found.map{ |f| f.tr('/', "\\") } else return found end else return nil end end # Iterate over each path glob the dir + program. path.split(File::PATH_SEPARATOR).each{ |dir| next unless File.exist?(dir) # In case of bogus second argument file = File.join(dir, program) # Dir[] doesn't handle backslashes properly, so convert them. Also, if # the program name doesn't have an extension, try them all. if MSWINDOWS file = file.tr("\\", "/") file += WIN32EXTS if File.extname(program).empty? end found = Dir[file].first # Convert all forward slashes to backslashes if supported if found && File.executable?(found) && !File.directory?(found) found.tr!(File::SEPARATOR, File::ALT_SEPARATOR) if File::ALT_SEPARATOR paths << found end } paths.empty? ? nil : paths.uniq end |
.which(program, path = ENV['PATH']) ⇒ Object
Looks for the first occurrence of program within path.
On Windows, it looks for executables ending with the suffixes defined in your PATHEXT environment variable, or ‘.exe’, ‘.bat’ and ‘.com’ if that isn’t defined, which you may optionally include in program.
Returns nil if not found.
Examples:
File.which('ruby') # => '/usr/local/bin/ruby'
File.which('foo') # => nil
101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 |
# File 'lib/ptools.rb', line 101 def self.which(program, path=ENV['PATH']) if path.nil? || path.empty? raise ArgumentError, "path cannot be empty" end # Bail out early if an absolute path is provided. if program =~ /^\/|^[a-z]:[\\\/]/i program += WIN32EXTS if MSWINDOWS && File.extname(program).empty? found = Dir[program].first if found && File.executable?(found) && !File.directory?(found) return found else return nil end end # Iterate over each path glob the dir + program. path.split(File::PATH_SEPARATOR).each{ |dir| dir = File.(dir) next unless File.exist?(dir) # In case of bogus second argument file = File.join(dir, program) # Dir[] doesn't handle backslashes properly, so convert them. Also, if # the program name doesn't have an extension, try them all. if MSWINDOWS file = file.tr("\\", "/") file += WIN32EXTS if File.extname(program).empty? end found = Dir[file].first # Convert all forward slashes to backslashes if supported if found && File.executable?(found) && !File.directory?(found) found.tr!(File::SEPARATOR, File::ALT_SEPARATOR) if File::ALT_SEPARATOR return found end } nil end |